
% yaw = yaw rotation for the coordinate frame that moves with the limb
% Rzo = Rotation matrix between the parent coordinate frame and the
% coordinate frame attached to the limb when the limb is in 'home position'
% ie. all rotations zero. This matrix tells us the zero offset.

% nearestja = joint angles of nearest neighbor - used to pick out yaw
% similarja = joint angles for previous frame, used for debugging.
% prevzyx = joint angles for previous frame in zyx ordering, used for debugging.

function [newjaxyz, newjazyx] = getangles_useyaw3(mpA, mpB, wRp, Rzo, nearestja, similarja, prevzyx)

abworld = [(mpB - mpA)/norm(mpB-mpA)]';
abparent = wRp' * abworld; % unit vector along the limb expressed in the parent
% coordinate frame

% Rja * Rzo* [0 0 1]' = abparent
% => R1 * [0 0 1]' = abparent
% R1 is in the zyx ordering, therefore gamma is about z-axis, beta is
% about y-axis and alpha is about x-axis
c = abparent;
beta1 = atan2(c(1) , sqrt(c(2)^2+c(3)^2)) * 180/pi;
alpha1 = atan2(-c(2)/cosd(beta1) , c(3)/cosd(beta1)) * 180/pi;
beta2 = atan2(c(1) , -sqrt(c(2)^2+c(3)^2)) * 180/pi;
alpha2 = atan2(-c(2)/cosd(beta2) , c(3)/cosd(beta2)) * 180/pi;

% use the nearest neighbor joint angles to determine yaw/gamma/rotation about z-axis
gammaknown = nearestja(1); betaknown = nearestja(2); alphaknown = nearestja(3);
Rknownxyz = getRxyz(gammaknown, betaknown, alphaknown);
R1known = Rknownxyz * Rzo;
% zyx
[g2,b2,a2, g1,b1,a1] = getRotationsZYX(R1known);

% possible combinations are [g1, beta1, alpha1],[g2, beta1, alpha1],[g1, beta2, alpha2],[g2, beta2, alpha2]
% choose gammanew to avoid wrapping. 
gammanew = g2;
betanew = beta1; alphanew = alpha1;

R1new = getRzyx(gammanew, betanew, alphanew);
Rja = R1new * Rzo';
[rX1, rY1, rZ1, rX2, rY2, rZ2 ] = getRotationsXYZ(Rja);

newjaxyz = [rX1, rY1, rZ1];
newjazyx = [gammanew, betanew, alphanew];

return

