% dots3d contains many motion captured sequences, merely concatenated together.
% In this function, each sequence is rotated and translated so that the first frame has zero
% rotation about the world z-axis (straight line from toes to head), and
% zero translation. These 'prepared' sequences are concatenated to make
% d3d.
% d2d is the projection of d3d onto the image plane specified by the user.
% We also take pieces of the motion captured sequences, and treat them as
% new sequences to artificially create more data.

function [d3d,d2d,corresja,R] = prepareData(dots3d,ja,skeleton)

d2d=[]; d3d=[]; corresja=[];
seqlength = [524 667]; % jumping jacks bob

sctr = 1;
ectr = seqlength(1);
for ctr = 1:length(seqlength)
    az = 92; el = 10; % jumping jacks. 
    % These angles are a good approximation to the camera in the artist's head. (see Section 3 of paper.)
    % We came up with these angles by plotting out the nearest motion
    % capture pose as a 3D matlab plot, and then rotating it around till
    % the projection looks like the hand drawn projection. Using the
    % command "[az,el] = view" , gives us these angles.
    disp(sprintf('[%d : %d]',sctr,ectr))
    [temp2d,temp3d,R] = project3dworldto2dimage_seqwise(dots3d(sctr:ectr,:),...
                        ja(sctr,:), az, el, skeleton.H, skeleton, skeleton.N);
    d2d = [d2d; temp2d];
    d3d = [d3d; temp3d];    
    corresja = [corresja; ja(sctr:ectr,:)]; 
    sctr = sum(seqlength(1:ctr)) + 1;
    if(ctr<length(seqlength)) ,ectr = sctr + seqlength(ctr+1) -1; end
end

% randomly pick out pieces of the original sequences, and treat them as new
% sequences. Rotate and translate them, and then concatenate with d3d.
for ctr = 1:50 %length(seqlength)
    az = 92; el = 10; % jumping jacks
    sctr = floor(rand*950)+1;
    ectr = sctr + 200; % we arbitrarily choose the length of the artificial sequences to be 200
    disp(sprintf('[%d : %d]',sctr,ectr))
    [temp2d,temp3d,R] = project3dworldto2dimage_seqwise(dots3d(sctr:ectr,:),...
                        ja(sctr,:), az, el, skeleton.H, skeleton, skeleton.N);
    d2d = [d2d; temp2d];
    d3d = [d3d; temp3d];    
    corresja = [corresja; ja(sctr:ectr,:)];
end