% Demo code for "Leveraging the Talent of Hand Animators to Create
% Three-Dimensional Animation", SCA 2009.

% Run this script by pressing F5, or typing liftoffdp4 at the command
% prompt in matlab.

skeletondescription;
[dots3d,ja] = getdatabase('bob');
% We want to know the 3d positions of the virtual markers when the joint
% angles are zero.
dotsHome = textread('../data/jj/bob/headbobdotshome.txt'); dotsHome = dotsHome(1,1:end-1);
%----
[d3d,d2d,corresja,R] = prepareData(dots3d,ja,skeleton);
query = gettestdata('bobmodified'); % one of the virtual markers was marked incorrectly and I fixed that datapoint in bobmodified.
%----
% Build a low dimensional subspace of poses (see Section 3.5, Equation 12)
[C,S,V,t2] = princomp(d3d);
c=C(:,1:20); % number of basis vectors , P=20
m3d = mean(d3d);
weightll = 0.001; % weight for the limblength error term, lambda_1 (see Section 3.5, Equation 5)
weightreg = 0.1; % weight for the regularization term, lambda_2 (see Section 3.5, Equation 5)
limblengths = getLimbLengths(dotsHome,H);
numDots = 24; % number of virtual markers
smoothingflag = 1; % to smooth or not to smooth the aligned poses
%----
% The user identifies a motion capture segment that is a reasonable match
% to the hand-drawn segment, in terms of the sequence of actions. (see
% Section 3, second paragraph)
% This reasonable match sequence is called result3d, or result2d, and the
% corresponding joint angles are called nearestja.
q = 1; fprintf(1,'query segment starts at %d.\n', q) 
numframesq = 137; 
numframesr = 128;
querysegment = query(q:q+numframesq-1,:);
result2d = d2d(524+105:524+105+numframesr-1, :); 
result3d = d3d(524+105:524+105+numframesr-1, :);
nearestja = corresja(524+105:524+105+numframesr-1, :);
%----
% The motion capture segment is time-warped to better align with the timing
% of the hand-drawn sequence. (see Section 3.3)
disp('Time Alignment ...')
[iq,ir] = dotimewarp_indicesonly(querysegment, result2d, result3d, skeleton);
% Now, it is valid to compare querysegment(iq,:) and result2d(ir,:) to see what
% poses need to be modified.
%----
% Here, the 2d poses are modified to better match the pose intended by the
% artist. (see Section 3.4)
disp('2D Pose alignment ...')
match = modifyinplane(querysegment(iq,:),result2d(ir,:), skeleton, smoothingflag);
%----
disp('3D Marker Reconstruction ...')
[recon2d, recon3d,llrecon,llerror] = reconstructLS(match, R, c, m3d, weightll, weightreg, limblengths, result3d(ir,:), skeleton, numDots);
% Uncomment the following line to play the time warped motion captured
% segment in red, and the time-and-pose-aligned reconstruction in blue. (see Figure 7, second and third row.)
playAll2in1(result3d(ir,:),recon3d,H);
%----
disp('Type "return" if you want to convert marker positions to joint angles, else type "dbquit".'); keyboard;
% Here, we convert the marker positions to joint angles. We also get the
% yaw rotation from the motion capture data. (see Section 3.6)
disp('Converting to joint angles ...')
[reconja] = convertdotsja(recon3d, nearestja(ir,:), dotsHome, skeleton, numDots);
%----
[norepiq, newr] = removerepetition(iq,1:size(reconja,1));
recon3dtw = recon3d(newr,:);
reconjatw = reconja(newr,:);
reconjatwsmoo5 = reconjatw;
% smooth out the feet.
for pii=[13:15 25:27]
    reconjatwsmoo5(:,pii) = smooth(reconjatwsmoo5(:,pii),5);
end
%----
% Now, watch these joint angles on a fully fleshed hierarchical skeletal
% model. Uncomment this if you have maya.
%seeScene(reconjatwsmoo5);






